#include "msgClient.hh"
#include<sstream>

#include "oro.h"
#include "oro_library.h"
#include "socket_connector.h"

#include<boost/regex.hpp>
#include "oroLexer.hh"

using namespace std;
using namespace oro;
using namespace boost;

static string NAME_OF_BRIDGE	= "";
static string MSGC_SERV 		= "";
static string ORO_SERV			= "";

static int MSGC_PORT 	= 5500;
static string ORO_PORT 	= "6969";
static int VERBOSE_MODE	= 0;

int main(int argc, char *argv[]){

	if(argc == 5) {
		NAME_OF_BRIDGE	= argv[1];
		MSGC_SERV		= argv[2];
		ORO_SERV		= argv[3];
		VERBOSE_MODE	= atoi(argv[4]);
	}
	else {
		cerr << "Args error. Usage: NAME_OF_BRIDGE MSGC_SERV ORO_SERV VERBOSE_MODE" << endl;
		exit(0);
	}

	// ----- Connection to msgconnector Server
	msgClient* mp = new msgClient();
	mp->connect(NAME_OF_BRIDGE, MSGC_SERV, MSGC_PORT);
	if(!mp->isConnected()) {
		cerr << "Unable to connect to msgconnector Server... exit." << endl;
		exit(0);
	}

	// ----- Connection To ORO/LIBORO Server

    Ontology* oro;
    try {
		SocketConnector connector(ORO_SERV, ORO_PORT);
    	oro = Ontology::createWithConnector(connector);
	
	// ----- Main loop - Message Transfer

		smatch what;
		while(mp->isConnected()) {
			pair<string, string> msg = mp->getBlockingMessage();
			if(VERBOSE_MODE == 1) cout << "[MSGC]Received from " << msg.first << ": " << msg.second << endl;
			string toAnalyse(msg.second);
			removeFormatting(toAnalyse);

			if(regex_match(toAnalyse, what, FindRequest)) { 
				set<Concept> OROresults;
				ostringstream statement, answer;
				statement << what[1] << " " << what[2] << " ?value";
				answer << "{\"ONTO-REP\": { \"VALUE\" : [";
				oro->find("value", statement.str(), OROresults);
				bool firstRes = false;
				for(set<Concept>::iterator it = OROresults.begin() ; it != OROresults.end() ; it++) {
					if(firstRes) answer << ", ";
					else firstRes = true;
					answer << "\"" << (*it).to_string() << "\"";
				}
				answer << "] }}";
				if(VERBOSE_MODE == 1) cout << "[MSGC]Sent to " << msg.first << ": " << answer.str() << endl;
				mp->sendMessage(msg.first, answer.str());
			}
			
			if(regex_match(toAnalyse, what, FindForAgentRequest)) { 
				set<Concept> OROresults;
				ostringstream statement, answer;
				statement << what[2] << " " << what[3] << " ?value";
				answer << "{\"ONTO-REP\": { \"VALUE\" : [";
				oro->findForAgent(what[1], "value", statement.str(), OROresults);
				bool firstRes = false;
				for(set<Concept>::iterator it = OROresults.begin() ; it != OROresults.end() ; it++) {
					if(firstRes) answer << ", ";
					else firstRes = true;
					answer << "\"" << (*it).to_string() << "\"";
				}
				answer << "] }}";
				if(VERBOSE_MODE == 1) cout << "[MSGC]Sent to " << msg.first << ": " << answer.str() << endl;
				mp->sendMessage(msg.first, answer.str());
			}	
		}
		cout << "is disconnected for MP" << endl;
		//connector.close();

    }
    catch (OntologyServerException ose) {
		cerr << "Unable to connect to ORO Server (OntologyServerException)... exit." << endl;
        exit(0);
    }
    catch (ConnectorException ose) {
		cerr << "Unable to connect to ORO Server (ConnectorException)... exit." << endl;
        exit(0);
    }

}
