#ifndef IMSG_HH
#define IMSG_HH

#include <cstdio>
#include <cstdlib>
#include <cstring>

class imsg {

	private:
		static const size_t max_size_ = 98304; // 96Ko
		static const size_t hl_ = 5;
		char data_[max_size_];
		size_t b_length_;

	public:
		imsg(): b_length_(0){}
		const char* data() const {return data_;}
		const size_t length() const {return b_length_ + hl_;}
		const size_t blength() {return b_length_;}
		const char* body() const {return data_ + hl_;}
		static size_t hsize() {return hl_;}
		static size_t msize() {return max_size_;}
		
		bool encode(const char *c) {
			b_length_ = std::strlen(c);
			if((b_length_ + hl_) > max_size_) return false;
			char header[hl_ + 1] = "";
			std::sprintf(header, "%5d", (int)b_length_);
			std::memcpy(data_, header, hl_);
			std::memcpy(data_ + hl_, c, b_length_);
			data_[this->length()] = '\0';
			return true;
		}
		
		void decodeHeader(const char *s) {
			std::memcpy(data_, s, hl_);
			data_[hl_] = '\0';
			b_length_ = std::atoi(data_);
		}
		
		void decodeBody(const char *s) {
			std::memcpy(data_ + hl_, s, b_length_);
		    data_[this->length()] = '\0';
		}

};

#endif 

